#ifndef SBCLIB_PALOPS
// Utilities for my 8-bit palettes, converting to/from 24-bit colours etc.
#include <stdio.h>
#include <rnd/sq.c>

void paldata666(float *r,float *g,float *b)
{ // Fills three float[256] arrays
	int n;
	for (n=0;n<16;n++)
	{
		r[n]=((n&4)>0)?0.5+0.5*((n&8)>0):0;
		g[n]=((n&2)>0)?0.5+0.5*((n&8)>0):0;
		b[n]=((n&1)>0)?0.5+0.5*((n&8)>0):0;
	}
	r[8]=g[8]=b[8]=0.25;
	for (n=16;n<40;n++)
		r[n]=g[n]=b[n]=(float)(n-16)/23;
	for (n=40;n<256;n++)
	{
		r[n]=0.2*((n-40)%6);
		g[n]=0.2*((n-40)/6%6);
		b[n]=0.2*((n-40)/36);
	}
}

void palto24bit(unsigned *ret)
{ // Fills ret[256] array
	float r[256],g[256],b[256];
	paldata666(r,g,b);
	for (int n=255;n>=0;n--) ret[n]=(((int)(r[n]*255.9))<<16)|(((int)(g[n]*255.9))<<8)|((int)(b[n]*255.9));
}

unsigned palcol(const int c)
{
	static unsigned pal[256],init=0;
	if (!init) {palto24bit(pal); init=1;}
	return pal[c<0?0:c>255?255:c];
}

unsigned char rgbto8bit(const unsigned c)
{
	float r=(c&0xFF0000)/(float)0xFF0000,g=(c&0xFF00)/(float)0xFF00,b=(c&0xFF)/(float)0xFF,dd,bd;
	static float pr[256],pg[256],pb[256]; static int init=0;
	if (!init) {paldata666(pr,pg,pb); init=1;}
	int n=(r>=0.25)*4+(g>=0.25)*2+(b>=0.25); unsigned char ret=n;
	bd=sq(r-pr[n])+sq(g-pg[n])+sq(b-pb[n]);
	n=8+(r>=0.5)*4+(g>=0.5)*2+(b>=0.5);
	if ((dd=sq(r-pr[n])+sq(g-pg[n])+sq(b-pb[n]))<bd) {ret=n; bd=dd;}
	n=16+(int)((r+g+b)*23/3+0.499);
	if ((dd=sq(r-pr[n])+sq(g-pg[n])+sq(b-pb[n]))<bd) {ret=n; bd=dd;}
	n=40+(int)(r*5+0.499)+(int)(g*5+0.499)*6+(int)(b*5+0.499)*36;
	if (sq(r-pr[n])+sq(g-pg[n])+sq(b-pb[n])<bd) ret=n;
	return ret;
}

void savejascpal(const float *r,const float *g,const float *b,const char *filename)
{
	FILE *out=fopen(filename,"wt");
	fprintf(out,"JASC-PAL\n0100\n256\n");
	for (int c=0;c<256;c++) fprintf(out,"%d %d %d\n",(int)(r[c]*255.9),(int)(g[c]*255.9),(int)(b[c]*255.9));
	fclose(out);
}

void paldata555(float *r,float *g,float *b)
{ // Old palette (NCGi?)
	int n;
	for (n=0;n<16;n++)
	{
		r[n]=((n&4)>0)?0.5+0.5*((n&8)>0):0;
		g[n]=((n&2)>0)?0.5+0.5*((n&8)>0):0;
		b[n]=((n&1)>0)?0.5+0.5*((n&8)>0):0;
	}
	r[8]=g[8]=b[8]=0.25;
	for (n=16;n<32;n++)
		r[n]=g[n]=b[n]=(float)(n-16)/15;
	for (n=32;n<128;n++)
	{
		float a=1,i,c=0,x=(n-32)/24;
		if (x==3) a=0.5;
		else if (x==1) c=0.5;
		else if (x==2) a=0.25;
		x=(n-32)%8; if (x>4) x=8-x; x/=4;
		i=a*x+c*(1.0-x);
		x=(n-32)/4%6;
		if (x==0) {r[n]=i; g[n]=c; b[n]=a;}
		else if (x==1) {r[n]=a; g[n]=c; b[n]=i;}
		else if (x==2) {r[n]=a; g[n]=i; b[n]=c;}
		else if (x==3) {r[n]=i; g[n]=a; b[n]=c;}
		else if (x==4) {r[n]=c; g[n]=a; b[n]=i;}
		else {r[n]=c; g[n]=i; b[n]=a;}
	}
	for (n=128;n<253;n++)
	{
		r[n]=0.25*((n-128)%5);
		g[n]=0.25*((n-128)/5%5);
		b[n]=0.25*((n-128)/25);
	}
	for (n=253;n<256;n++) r[n]=g[n]=b[n]=0;
}

#define SBCLIB_PALOPS
#endif
